from django.contrib.auth.models import (
    AbstractBaseUser,
    BaseUserManager,
    PermissionsMixin,
)
from django.core.exceptions import ValidationError
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone

from dynamic_filenames import FilePattern
from stdimage.models import StdImageField


class UserManager(BaseUserManager):
    use_in_migrations = True

    def create_user(self, email, password=None, **extra_fields):
        """
        Crée et enregistre un compte utilisat⋅eur⋅rice avec l'adresse mail et le
        mot de passe donnés.
        """
        extra_fields.setdefault('is_superuser', False)
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, password, **extra_fields):
        """Crée et enregistre un compte super-utilisat⋅eur⋅rice."""
        extra_fields.setdefault('is_superuser', True)
        if extra_fields.get('is_superuser') is not True:  # pragma: no cover
            raise ValueError(
                "Un⋅e super-utilisat⋅eur⋅rice doit avoir is_superuser=True."
            )
        return self.create_user(email, password, **extra_fields)


class User(AbstractBaseUser, PermissionsMixin):
    """
    Les seuls éléments strictement nécessaires au fonctionnement de
    **Bénévalibre** sont une adresse courriel valide, ainsi qu'un des trois
    éléments suivants :

    * nom,
    * prénom,
    * peudonyme.

    L'adresse courriel est utilisée pour l'authentification.

    .. note::

        Ces éléments d'identité seront a priori les seuls par lesquels vous
        vous rendrez reconnaissable au sein de l'application. Veillez à être
        suffisamment précis pour rester reconnaissable par les personnes avec
        lesquelles vous travaillerez.

    Vous pouvez également ajouter un avatar à votre profil. Si vous n'en
    fournissez pas, un avatar par défaut sera créé.

    Les éléments renseignés seront visibles des personnes avec lesquelles vous
    interagirez via l'application.
    """

    # Identité
    first_name = models.CharField("prénom", max_length=30, blank=True)
    last_name = models.CharField("nom", max_length=50, blank=True)
    pseudo = models.CharField("pseudo", max_length=30, blank=True)
    email = models.EmailField(
        "adresse mail",
        unique=True,
        error_messages={
            'unique': "Un compte avec cette adresse mail existe déjà."
        },
    )

    # Permissions
    is_active = models.BooleanField(
        "actif",
        default=True,
        help_text=(
            "Détermine si le compte utilisat⋅eur⋅rice doit être considéré "
            "comme actif dans l'application. Vous pouvez le désactiver plutôt "
            "que de le supprimer."
        ),
    )

    # Méta-données
    date_joined = models.DateTimeField(
        "date d'inscription", default=timezone.now, editable=False
    )

    # Avatar
    avatar_upload_to = FilePattern(filename_pattern='avatars/{uuid:x}{ext}')
    avatar = StdImageField(
        "avatar",
        upload_to=avatar_upload_to,
        blank=True,
        null=True,
        variations={'thumbnail': {'width': 48, 'height': 48, 'crop': True}},
    )

    objects = UserManager()

    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = "compte utilisat⋅eur⋅rice"
        verbose_name_plural = "comptes utilisat⋅eurs⋅rices"
        ordering = ('last_name', 'first_name', 'pseudo')

    @property
    def is_staff(self):
        # Détermine si le compte a accès à l'administration de Django, qu'on
        # vise à ne pas utiliser dans l'application.
        return self.is_superuser

    @property
    def is_manager(self):
        return any(
            [e.role.manage_association for e in self.engagement_set.all()]
        )

    @property
    def pseudo_rand_color(self):
        return self.date_joined.microsecond % 360

    def __str__(self):
        return self.full_name or self.email

    def clean(self):
        """Vérifie les champs d'identité et normalise l'email."""
        super().clean()

        if not any([self.first_name, self.last_name, self.pseudo]):
            raise ValidationError(
                "Au moins le prénom, le nom ou le pseudo est requis.",
                code='name_required',
            )

        self.email = self.__class__.objects.normalize_email(self.email)

    def get_full_name(self):
        """Retourne le prénom et le nom de la personne, et/ou son pseudo."""
        if not self.first_name and not self.last_name:
            return self.pseudo
        full_name = "{} {}".format(self.first_name, self.last_name).strip()
        if self.pseudo:
            # ajoute le pseudo entre parenthèses
            full_name += " ({})".format(self.pseudo)
        return full_name

    full_name = property(get_full_name)

    def get_short_name(self):
        """Retourne le prénom, le nom ou le pseudo de la personne."""
        return self.first_name or self.last_name or self.pseudo

    short_name = property(get_short_name)

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Envoi un courriel à cette personne."""
        send_mail(subject, message, from_email, [self.email], **kwargs)
