import re

from django.contrib.auth import get_user_model
from django.contrib.messages import SUCCESS as SUCCESS_LEVEL
from django.contrib.messages import get_messages
from django.urls import reverse, reverse_lazy

import pytest
from bs4 import BeautifulSoup as bs

from benevalibre.utils.tests import count_text_in_content

User = get_user_model()

RE_URL = re.compile(r'http(|s)://[^/]*/([^ ]*/)$', re.MULTILINE)


@pytest.mark.django_db
class TestPasswordReset:
    password_reset_url = reverse_lazy('accounts:password_reset')

    @pytest.mark.parametrize(
        'host_name, instance_name',
        [
            ('testserver', 'testserver'),
            ('www.xn--test-accentu-meb.eu', 'www.test-accentué.eu'),
        ],
    )
    def test_workflow(
        self, settings, client, mailoutbox, host_name, instance_name
    ):
        """
        Vérifie le parcours pour réinitialiser son mot de passe.
        """
        settings.ALLOWED_HOSTS = [host_name]
        client.defaults['HTTP_HOST'] = host_name

        user_email = 'iforgot@no.lan'

        # 0. voila un⋅e utilisat⋅eur⋅rice sans mot de passe
        User.objects.create_user(email=user_email, password='')
        assert User.objects.exclude(pseudo='instance').count() == 1

        response = client.get(reverse('accounts:login'))
        assert response.status_code == 200
        assert len(list(get_messages(response.wsgi_request))) == 0

        # 1. si on demande pour un⋅e utilisat⋅eur⋅rice existant ...
        response = client.post(
            self.password_reset_url, data={'email': user_email}, follow=True
        )
        # ... on est redirigé
        assert response.status_code == 200
        assert response.redirect_chain[-1][0] == reverse('accounts:login')
        # ... et on a un courriel.
        assert len(mailoutbox) == 1
        # ... avec un contenu qui a du sens
        assert (
            mailoutbox[0].subject == "Réinitialisation de votre "
            "mot de passe sur " + instance_name
        )
        assert "de votre compte sur " + instance_name in mailoutbox[0].body

        # 3. si on suit le lien reçu dans le courriel...
        confirm_url = RE_URL.search(mailoutbox[0].body).group()
        assert confirm_url
        # ... il nous renvoi vers un formulaire
        response = client.get(confirm_url, follow=True)
        form_url = response.request['PATH_INFO']
        assert form_url
        # ... qu'on soumet avec le nouveau mot de passe
        response = client.post(
            form_url,
            data={'new_password1': 'sho8sho8', 'new_password2': 'sho8sho8'},
            follow=True,
        )
        assert response.status_code == 200
        # ... et on verifie que ce nouveau mot de passe est bien valide
        assert client.login(username=user_email, password='sho8sho8')
        # ... avant de se déconnecter.
        response = client.get(reverse('accounts:logout'), follow=True)
        assert response.status_code == 200

        # 4. on verifie que le lien n'est plus valide
        response = client.get(confirm_url, follow=True)
        form_url = response.request['PATH_INFO']
        assert form_url
        # ... si on soumet le formulaire avec un nouveau mot de passe
        response = client.post(
            form_url,
            data={'new_password1': 'sho9sho9', 'new_password2': 'sho9sho9'},
        )
        assert response.status_code == 200
        assert not client.login(username=user_email, password='sho9sho9')


@pytest.mark.django_db(transaction=True)
class TestRegistration:
    registration_url = reverse_lazy('accounts:registration_register')
    resend_url = reverse_lazy('accounts:registration_resend_activation')

    @pytest.mark.parametrize(
        'host_name, instance_name',
        [
            ('testserver', 'testserver'),
            ('www.xn--test-accentu-meb.eu', 'www.test-accentué.eu'),
        ],
    )
    def test_register_workflow(
        self, settings, client, mailoutbox, host_name, instance_name
    ):
        """
        Vérifie le parcours de création d'un compte.
        """
        settings.ALLOWED_HOSTS = [host_name]
        client.defaults['HTTP_HOST'] = host_name

        user_email = 'register@no.lan'

        # 0. on affiche le formulaire
        response = client.get(self.registration_url)
        assert response.status_code == 200
        assert 'form' in response.context

        # 1. on soumet le formulaire...
        data = {
            'email': user_email,
            'password1': 'sho7sho7',
            'password2': 'sho7sho7',
        }
        response = client.post(self.registration_url, data)
        assert response.status_code == 200
        assert response.context['form'].errors
        assert count_text_in_content(response, "Au moins le prénom, le nom")
        # ... avec au moins un pseudo
        data['pseudo'] = 'Toto'
        response = client.post(self.registration_url, data, follow=True)
        assert response.status_code == 200
        assert response.redirect_chain[-1][0] == reverse(
            'accounts:registration_complete'
        )
        # ... un⋅e utilisat⋅eur⋅rice inactif est créé
        user = User.objects.get(email=user_email)
        assert user.check_password('sho7sho7')
        assert user.is_active is False
        assert not client.login(username=user_email, password='sho7sho7')
        # ... et on a un courriel avec un lien d'activation.
        assert len(mailoutbox) == 1
        # ... avec un contenu qui a du sens
        assert (
            mailoutbox[0].subject
            == "Activation de votre compte sur " + instance_name
        )
        assert (
            "sur " + instance_name + ". Si ce n'était pas vous, "
            in mailoutbox[0].body
        )
        first_activation_url = RE_URL.search(mailoutbox[0].body).group()
        assert first_activation_url

        # 2. on peut demander le renvoi du courriel
        client.post(self.resend_url, data={'email': user_email}, follow=True)
        # ... le lien d'activation est régénéré
        assert len(mailoutbox) == 2
        activation_url = RE_URL.search(mailoutbox[1].body).group()
        assert activation_url and activation_url != first_activation_url

        # 3. si on suit le lien d'activation...
        response = client.get(activation_url, follow=True)
        # ... on est redirigé à la page de connexion
        assert response.status_code == 200
        assert response.redirect_chain[-1][0] == reverse('accounts:login')
        # ... avec un message
        messages = list(get_messages(response.wsgi_request))
        assert len(messages) == 1
        assert messages[0].level == SUCCESS_LEVEL
        assert user_email in messages[0].message
        # ... et l'utilisat⋅eur⋅rice est désormais actif.
        user.refresh_from_db()
        assert user.is_active is True
        assert client.login(username=user_email, password='sho7sho7')

    def test_resend_email_unknown(self, client, mailoutbox):
        user_email = 'idontexist@no.lan'

        # si on demande pour utilisat⋅eur⋅rice inexistant...
        response = client.post(
            self.resend_url, data={'email': user_email}, follow=True
        )
        # ... on est redirigé à la page de connexion
        assert response.status_code == 200
        assert response.redirect_chain[-1][0] == reverse('accounts:login')
        # ... avec un message
        messages = list(get_messages(response.wsgi_request))
        assert len(messages) == 1
        assert messages[0].level == SUCCESS_LEVEL
        assert user_email in messages[0].message
        # ... mais on a pas de courriel.
        assert len(mailoutbox) == 0

    def test_closed(self, client, settings):
        settings.REGISTRATION_OPEN = False

        # vérifie la redirection
        response = client.get(self.registration_url)
        assert response.status_code == 302
        assert response['Location'] == reverse('accounts:registration_closed')
        # ... et l'affichage de la page
        response = client.get(self.registration_url, follow=True)
        assert response.status_code == 200


@pytest.mark.django_db
class TestPasswordChange:
    url = reverse('accounts:password_change')

    def test_workflow(self, client, user):
        user.set_password('password')
        user.save()

        assert client.login(username=user.email, password='password')

        response = client.get(self.url)
        assert response.status_code == 200

        form = bs(response.content, 'html.parser').find('form')
        input_kv = [
            (i.get('name'), i.get('value'))
            for i in form.find_all('input')
            if i.get('value')
        ]
        data = dict(input_kv)
        data['old_password'] = 'password'
        data['new_password1'] = 'sho9sho9'
        data['new_password2'] = 'sho9sho9'

        response = client.post(self.url, data, follow=True)
        assert response.status_code == 200

        messages = list(get_messages(response.wsgi_request))
        assert len(messages) == 1
        assert messages[0].level == SUCCESS_LEVEL

        assert client.login(username=user.email, password='sho9sho9')

    def test_unknow_user_password_reset_failed(self, client):
        response = client.post(
            reverse('accounts:password_reset'),
            {'email': 'jonathan@no.lan'},
        )
        assert "est associé à aucun compte." in response.rendered_content

    def test_unactivated_user_password_reset_failed(self, client):
        User.objects.create_user(
            email='christopher@no.lan',
            password='sho7sho7',
            is_active=False,
        )
        assert User.objects.exclude(pseudo='instance').count() == 1
        response = client.post(
            reverse('accounts:password_reset'),
            {'email': 'christopher@no.lan'},
            follow=True,
        )
        assert (
            reverse('accounts:registration_resend_activation')
            in response.rendered_content
        )


@pytest.mark.django_db
class TestUserUpdate:
    url = reverse('accounts:update')

    def test_unauthorized(self, client):
        response = client.get(self.url)
        assert response.status_code == 302

    def test_update(self, client, user):
        client.force_login(user)

        response = client.get(self.url)
        assert response.status_code == 200

        form = bs(response.content, 'html.parser').find('form')
        input_kv = [
            (i.get('name'), i.get('value'))
            for i in form.find_all('input')
            if i.get('value')
        ]
        data = dict(input_kv)
        data['pseudo'] = "Pseudo Nymous"

        response = client.post(self.url, data, follow=True)
        assert response.status_code == 200

        messages = list(get_messages(response.wsgi_request))
        assert len(messages) == 1
        assert messages[0].level == SUCCESS_LEVEL

        user.refresh_from_db()
        assert user.pseudo == "Pseudo Nymous"
