from django.contrib import messages
from django.contrib.auth import views as auth_views
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.messages.views import SuccessMessageMixin
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy
from django.utils.html import format_html
from django.views.generic.base import TemplateView
from django.views.generic.edit import UpdateView

from registration.backends.default import views as registration_views

from benevalibre.utils.views import PageMixin

from .forms import (
    AuthenticationForm,
    PasswordChangeForm,
    PasswordResetForm,
    RegistrationForm,
    ResendActivationForm,
    SetPasswordForm,
    UserUpdateForm,
)
from .models import User

LOGIN_URL = reverse_lazy('accounts:login')
HOME_URL = reverse_lazy('base:home')


# AUTHENTICATION AND PASSWORD RESET
# ------------------------------------------------------------------------------


class LoginView(PageMixin, auth_views.LoginView):
    """
    Pour accèder à **Bénévalibre**, il est indispensable d'avoir un compte
    utilisateur. Si vous n'en avez pas, vous pouvez en créer un en suivant le
    lien :guilabel:`Créer mon compte…` sur la page de connexion. Un courriel
    vous sera envoyé à l'adresse indiquée et contiendra un lien nécessaire à
    l'activation de votre compte.
    """

    form_class = AuthenticationForm
    template_name = 'accounts/login.html'
    title = "Connexion"


class LogoutView(auth_views.LogoutView):
    def dispatch(self, request, *args, **kwargs):
        response = super().dispatch(request, *args, **kwargs)
        messages.success(request, "Vous avez été déconnecté.")
        return response


class PasswordResetView(PageMixin, auth_views.PasswordResetView):
    """
    Si vous avez oublié votre mot de passe, il est possible d'en définir un
    nouveau. Pour cela, lorsque vous êtes sur la page de connexion, cliquez sur
    le lien :guilabel:`Mot de passe oublié ?`. Votre adresse mail vous sera
    demandée, entrez celle qui est définie pour votre compte utilisateur. Si
    celle-ci est correcte, vous recevrez alors un courriel vous expliquant la
    procédure à suivre pour définir votre nouveau mot de passe.
    """

    form_class = PasswordResetForm
    success_url = LOGIN_URL
    template_name = 'accounts/password_reset.html'
    email_template_name = 'accounts/password_reset_email.txt'
    html_email_template_name = 'accounts/password_reset_email.html'
    subject_template_name = 'accounts/password_reset_subject.txt'
    title = "Réinitialiser mon mot de passe"

    def form_valid(self, form):
        email = form.cleaned_data['email']
        try:
            user = User.objects.get(email=email)
        except User.DoesNotExist:
            form.add_error(None, "Cet email n'est associé à aucun compte.")
            return self.render_to_response(self.get_context_data(form=form))
        if not user.is_active:
            form.add_error(
                None,
                format_html(
                    "Cet utilisateur est bien enregistré mais il n'a pas été "
                    "activé. Sachez que vous pouvez demander le <a href='{}'> "
                    "renvoi d'un lien d'activation.</a>".format(
                        reverse_lazy('accounts:registration_resend_activation')
                    )
                ),
            )
            return self.render_to_response(self.get_context_data(form=form))
        messages.success(
            self.request,
            format_html(
                (
                    "Un courriel a été envoyé à <i>{}</i>, si cette adresse "
                    "mail est valide. Il contient les instructions à suivre "
                    "pour réinitialiser votre mot de passe."
                ),
                email,
            ),
        )
        return super().form_valid(form)


class PasswordResetConfirmView(PageMixin, auth_views.PasswordResetConfirmView):
    form_class = SetPasswordForm
    success_url = LOGIN_URL
    template_name = 'accounts/password_reset_confirm.html'
    title = "Réinitialiser mon mot de passe"

    def get_success_url(self):
        messages.success(
            self.request, "Votre mot de passe a été réinitialisé."
        )
        return super().get_success_url()


class PasswordChangeView(PageMixin, auth_views.PasswordChangeView):
    form_class = PasswordChangeForm
    success_url = HOME_URL
    template_name = 'accounts/password_change.html'
    title = "Modifier mon mot de passe"

    def get_success_url(self):
        messages.success(self.request, "Votre mot de passe a été modifié.")
        return super().get_success_url()


# PROFILE
# ------------------------------------------------------------------------------


class UserUpdateView(
    LoginRequiredMixin, PageMixin, SuccessMessageMixin, UpdateView
):
    form_class = UserUpdateForm
    success_url = HOME_URL
    success_message = "Votre profil a été mis à jour."
    template_name = 'accounts/user_update.html'
    title = "Mettre à jour mon profil"

    def get_object(self):
        return User.objects.get(pk=self.request.user.pk)


# REGISTRATION
# ------------------------------------------------------------------------------


class RegistrationView(PageMixin, registration_views.RegistrationView):
    form_class = RegistrationForm
    disallowed_url = reverse_lazy('accounts:registration_closed')
    success_url = reverse_lazy('accounts:registration_complete')
    title = "Créer un compte"


class RegistrationCompleteView(PageMixin, TemplateView):
    template_name = 'registration/registration_complete.html'
    title = "Créer un compte"


class RegistrationClosedView(PageMixin, TemplateView):
    template_name = 'registration/registration_closed.html'
    title = "Inscription indisponible"


class ActivationView(registration_views.ActivationView):
    def get_success_url(self, user):
        messages.success(
            self.request,
            format_html(
                (
                    "Votre compte est désormais activé ! Vous pouvez "
                    "maintenant vous connecter avec votre adresse <i>{}</i>."
                ),
                user.email,
            ),
        )
        return LOGIN_URL


class ResendActivationView(PageMixin, registration_views.ResendActivationView):
    form_class = ResendActivationForm
    title = "Renvoyer le courriel d'activation"

    def render_form_submitted_template(self, form):
        messages.success(
            self.request,
            format_html(
                (
                    "Un courriel a été envoyé à <i>{}</i>, si cette adresse "
                    "mail est valide. Il contient un nouveau lien permettant "
                    "d'activer votre compte."
                ),
                form.cleaned_data['email'],
            ),
        )
        return HttpResponseRedirect(LOGIN_URL)
