from datetime import date, timedelta

from django.contrib.messages import get_messages
from django.urls import reverse, reverse_lazy

import pytest
from bs4 import BeautifulSoup as bs

from benevalibre.accounts.models import User
from benevalibre.association.models import ActivityField, Association, Category
from benevalibre.base.tables import format_duration
from benevalibre.utils.tests import (
    RE_DELETE_BTN,
    count_text_in_content,
    is_login_view,
)


class TestHome:
    url = reverse_lazy('base:home')

    @pytest.mark.django_db
    def test_anonymous(self, client):
        response = client.get(self.url)
        assert response.status_code == 200
        assert count_text_in_content(response, reverse('association:index'))

    @pytest.mark.django_db
    def test_user_redirect(self, client, user):
        client.force_login(user)

        response = client.get(self.url)
        assert response.status_code == 302
        assert response.url == reverse('base:board')


@pytest.mark.django_db
class TestAssociationEngage:
    board_url = reverse_lazy('base:board')

    @pytest.fixture(autouse=True)
    def setup_association(self, association):
        self.association = association
        self.url = reverse('base:association:engage', args=[association.id])

    def test_get_redirect(self, client, user):
        client.force_login(user)
        response = client.get(self.url, follow=True)
        assert response.resolver_match.view_name == 'association:index'
        assert not self.association.get_engagement(user)

    def test_engage(self, client, user):
        assert self.association.can_self_engage(user)

        client.force_login(user)
        response = client.post(self.url, {}, follow=True)
        assert response.status_code == 200
        assert self.association.get_engagement(user)
        assert count_text_in_content(response, "Mes engagements associatifs")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "modération" not in messages[0]
        assert "acceptée" in messages[0]

    def test_association_engagement_twice(self, client, user):
        client.force_login(user)
        response = client.post(self.url, {}, follow=True)
        response = client.post(self.url, {}, follow=True)
        assert response.status_code == 200
        assert self.association.get_engagement(user)
        assert response.resolver_match.view_name == 'association:index'

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "déjà engagé dans cette association" in messages[0]

    def test_moderated_association_engagement(
        self, client, mailoutbox, user, engagement, foreign
    ):
        self.association.moderate_engagement = True
        self.association.save()
        engagement.role = self.association.role_set.get_dirigeant()
        engagement.save()

        assert not self.association.can_self_engage(user)

        # 1. on fait la demande d'engagement
        client.force_login(foreign)
        response = client.post(self.url, {}, follow=True)
        assert response.status_code == 200

        # 2. elle est suspendue en modération
        engagements_qs = foreign.engagement_set.filter(
            association=self.association
        )
        assert len(engagements_qs) == 1
        assert not engagements_qs.active().exists()
        assert count_text_in_content(response, "Mes engagements associatifs")
        assert count_text_in_content(response, "Enregistrer une association")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "modération" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "L'AssoTest" in str(table_row[0])
        assert 'href="#"' in str(table_row[0])
        assert "disabled" in str(table_row[0])
        assert "En modération" in str(table_row[0])

        # 3. elle est visible à un modérat⋅eur⋅rice
        client.logout()
        client.force_login(user)
        response = client.get(self.board_url)
        assert count_text_in_content(response, "Modérations de bénévoles")

        # Qui reçoit une notification
        assert len(mailoutbox) == 1
        assert user.email in mailoutbox[0].to
        assert user.get_short_name() in mailoutbox[0].body
        assert self.association.name in mailoutbox[0].body
        assert "Validation d'un nouvel engagement" in mailoutbox[0].subject

        # 4. on donne suite
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        data = {'role': self.association.role_set.get_benevole().id}
        response = client.post(url, data, follow=True)
        response = client.get('', follow=True)
        assert not count_text_in_content(response, "Modérations de bénévoles")

        # 5. Le bénévole est notifié
        assert len(mailoutbox) == 2
        assert foreign.email in mailoutbox[1].to
        assert foreign.get_short_name() in mailoutbox[1].body
        assert self.association.name in mailoutbox[0].body
        assert "Mise à jour de votre engagement" in mailoutbox[1].subject

        # 6. le bénévole a désormais accès
        client.logout()
        client.force_login(foreign)
        response = client.get(self.board_url)

        assert count_text_in_content(response, "Mes engagements associatifs")
        assert count_text_in_content(response, "Enregistrer une association")

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "L'AssoTest" in str(table_row[0])
        assert "disabled" not in str(table_row[0])


@pytest.mark.django_db
class TestBoard:
    url = reverse_lazy('base:board')

    def test_get_anonymous(self, client):
        response = client.get(self.url, follow=True)
        assert response.status_code == 200
        assert is_login_view(response)

    def test_get_user_not_engaged(self, client, user):
        client.force_login(user)

        # the user is not engaged by those associations
        Association.objects.create(name='bling', conceal=True)
        Association.objects.create(name='blang')

        response = client.get(self.url)
        assert response.status_code == 200
        assert count_text_in_content(response, "Aucune donnée disponible")
        assert count_text_in_content(response, "Mes engagements associatifs")
        assert count_text_in_content(response, "Enregistrer une association")

    def test_get_user_engaged(self, client, association, engagement, user):
        client.force_login(user)

        response = client.get(self.url)
        assert response.status_code == 200
        assert count_text_in_content(response, "AssoTest")

    @pytest.mark.parametrize(
        'use, expected', [(None, 404), ('user', 200), ('staff', 404)]
    )
    def test_create_associated_self_benevalo_access(
        self, client, association, engagement, user, superuser, use, expected
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse(
                'base:association:self-benevalo:create', args=[association.id]
            )
        )
        assert response.status_code == expected

    def test_create_update_delete_associated_self_benevalo(
        self, client, association, category, engagement, user, superuser
    ):
        client.force_login(user)

        # 0. On fetch les engagements
        response = client.get(self.url)
        assert response.status_code == 200

        # 1. on créé
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='create_benevalo-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)
        assert not count_text_in_content(response, "level")
        assert not count_text_in_content(response, "project")

        data = {
            'user': user.id,
            'title': 'bénévalo de test',
            'duration_1': '33',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        association.benevalo_set.get(user=user, title='bénévalo de test')

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

        # 2. On a la liste

        table = str(bs(response.content, 'html.parser').find('tbody'))

        assert "33 minutes" in table
        assert "bénévalo de test" in table

        # 3. On update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a')[-1]
            .attrs['href']
        )
        data = {
            'user': user.id,
            'title': 'bénévalo de lutte',
            'duration_1': '42',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table = str(bs(response.content, 'html.parser').find('tbody'))
        assert "33 minutes" not in table
        assert "bénévalo de test" not in table
        assert "42 minutes" in table
        assert "bénévalo de lutte" in table

        # 4. on vérifie la date dans le formulaire  (BUG #20)
        response = client.get(url, follow=True)
        execution_date = (
            bs(response.content, 'html.parser')
            .find(id='id_execution_date')
            .attrs['value']
        )
        assert execution_date == date.today().strftime('%Y-%m-%d')

        # 5. On delete
        url = (
            bs(response.content, 'html.parser')
            .find('a', string=RE_DELETE_BTN)
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "» a été supprimé" in messages[0]

        table = str(bs(response.content, 'html.parser').find('tbody'))
        assert "Aucune donnée disponible." in table

    def test_create_associated_benevalo_with_one_level(
        self, client, association, level, category, engagement, user, superuser
    ):
        client.force_login(user)

        # 0. On fetch les engagements
        response = client.get(self.url)
        assert response.status_code == 200

        # 1. on créé
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='create_benevalo-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)
        assert count_text_in_content(response, "level")
        assert not count_text_in_content(response, "project")

        data = {
            'user': user.id,
            'title': 'BenevaloTest',
            'duration_1': '33',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }

        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        association.benevalo_set.get(user=user, title="BenevaloTest")

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

    def test_create_associated_benevalo_with_one_project(
        self,
        client,
        association,
        project,
        category,
        engagement,
        user,
        superuser,
    ):
        client.force_login(user)

        # 0. On fetch les engagements
        response = client.get(self.url)
        assert response.status_code == 200

        # 1. on créé
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='create_benevalo-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)
        assert count_text_in_content(response, "project")
        assert not count_text_in_content(response, "level")

        data = {
            'user': user.id,
            'title': 'test',
            'duration_1': '33',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }

        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        association.benevalo_set.get(user=user, title='test')

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été ajouté" in messages[0]

    def test_create_associated_benevalo_with_moderation(
        self, client, mailoutbox, association, category, manager, user, foreign
    ):
        association.moderate_benevalo = True
        association.save()

        engagement = association.engagement_set.create(
            user=foreign, role=association.role_set.get_benevole()
        )

        # 0. On fetch les engagements
        client.force_login(foreign)
        response = client.get(self.url)
        assert response.status_code == 200

        assert not count_text_in_content(response, "level")

        # 1. On fait la saisie
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='create_benevalo-link')[-1]
            .attrs['href']
        )
        data = {
            'user': user.id,
            'title': 'BenevaloTest',
            'duration_1': '33',
            'distance': '102',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        benevalo = association.benevalo_set.get(
            user=foreign, title='BenevaloTest'
        )
        assert not benevalo.is_active

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "modération" in messages[0]

        # 2. Elle est visible mais suspendue en modération
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert "(en modération)" in str(table_row[0])

        # 3. Elle est visible à un modérat⋅eur⋅rice
        client.logout()
        client.force_login(user)
        for urlname, args in [
            ('base:board', []),
            ('base:association:benevalo:index', [association.id]),
            (
                'base:association:engagement:benevalo',
                [association.id, engagement.id],
            ),
        ]:
            response = client.get(reverse(urlname, args=args))
            assert count_text_in_content(
                response, "Modérations de bénévolats en attente"
            )
            table_row = (
                bs(response.content, 'html.parser')
                .find('tbody')
                .find_all('tr')
            )
            assert len(table_row) == 1
            assert "foreign@example.org" in str(table_row[0])
            assert "BenevaloTest" in str(table_row[0])
            assert "Voir" in str(table_row[0])

        # Qui reçoit une notification
        assert len(mailoutbox) == 1
        assert user.email in mailoutbox[0].to
        assert user.get_short_name() in mailoutbox[0].body
        assert association.name in mailoutbox[0].body
        assert "Validation d'une nouvelle action" in mailoutbox[0].subject

        # 4. On donne suite
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a', class_='update-link')[-1]
            .attrs['href']
        )
        response = client.get(url, follow=True)

        # On prend tout et on post
        form = bs(response.content, 'html.parser').find("form")
        input_kv = [
            (i.get('name'), i.get('value'))
            for i in form.find_all('input')
            if i.get('value')
        ]
        select_kv = [
            (s.parent.get('name'), s.get('value'))
            for s in form.find_all('option', attrs={"selected": ""})
            if s.get('value')
        ]
        data = dict(input_kv + select_kv)

        response = client.post(url, data, follow=True)

        assert not count_text_in_content(
            response, "Modérations de bénévolats en attente"
        )

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        # 5. Le bénévole a désormais vue sur une saisie normale
        client.logout()
        client.force_login(foreign)
        response = client.get(
            reverse(
                'base:association:self-benevalo:index', args=[association.id]
            )
        )
        assert response.status_code == 200

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert "(en modération)" not in str(table_row[0])
        assert "33 minutes" in str(table_row[0])
        assert "BenevaloTest" in str(table_row[0])


@pytest.mark.django_db
class TestUserList:
    @pytest.mark.parametrize(
        'use, expected',
        [
            (None, 404),
            ('user', 302),
            ('animateur', 200),
            ('staff', 404),
            ('foreign', 404),
        ],
    )
    def test_get_users(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use == 'animateur':
            animateur = User.objects.create_user(
                email="test@email.com",
            )
            role = association.role_set.create(
                name="Animateur", list_users=True
            )
            animateur.engagement_set.create(association=association, role=role)

            client.force_login(animateur)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('base:association:user', args=[association.id])
        )
        assert response.status_code == expected

        if response.status_code == 302:
            assert response['Location'] == reverse(
                'association:engagement:index', args=[association.id]
            )

        if response.status_code == 200:

            table_row = (
                bs(response.content, 'html.parser')
                .find('tbody')
                .find_all('tr')
            )
            assert len(table_row) == 2
            assert "Animateur" in str(table_row[-1])
            assert "test@email.com" in str(table_row[-1])
            # table à 3 colonnes sans les actions
            assert len(table_row[-1].find_all(['th', 'td'])) == 3


@pytest.mark.django_db
class TestBenevalo:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse('base:association:benevalo:index', args=[association.id])
        )
        assert response.status_code == expected

    def test_index_table_footer(
        self, client, association, category, user, engagement
    ):
        user.benevalo_set.create(
            title="test 1",
            duration=timedelta(0, 1260),
            distance=21,
            execution_date=date.today(),
            association=association,
            category=category,
        )

        user.benevalo_set.create(
            title="test 2",
            duration=timedelta(0, 1260),
            distance=21,
            execution_date=date.today() - timedelta(1),
            association=association,
            category=category,
        )

        client.force_login(user)
        response = client.get(
            reverse(
                'base:association:self-benevalo:index', args=[association.id]
            )
        )
        assert response.status_code == 200
        table_foot = bs(response.content, 'html.parser').find('tfoot')
        assert "42 minutes" in str(table_foot)
        assert "42,00 km" in str(table_foot)
        assert "2 éléments" in str(table_foot)
        assert "sur 1 jour" in str(table_foot)

    def test_index_table_export(
        self, client, association, project, level, category, user, engagement
    ):
        user.benevalo_set.create(
            title="test 1",
            duration=timedelta(0, 1260),
            distance=21,
            execution_date=date.today(),
            association=association,
            category=category,
        )

        user.benevalo_set.create(
            title="test 2",
            duration=timedelta(0, 1260),
            distance=21,
            execution_date=date.today() - timedelta(1),
            execution_end_date=date.today() + timedelta(1),
            association=association,
            category=category,
            level=level,
            project=project,
            description='mise sous plis des convocations',
            is_active=False,
        )

        client.force_login(user)
        response = client.get(
            reverse(
                'base:association:self-benevalo:index', args=[association.id]
            )
            + '?_export=json'
        )
        assert response.status_code == 200
        assert response.json() == [
            {
                'Titre': "test 1",
                'Durée': "21 minutes",
                'Heures, en décimal': 0.35,
                'Distance': 21,
                'Date': date.today().isoformat(),
                'Association': str(association),
                'Bénévole': str(user),
                'Catégorie': str(category),
                'Niveau': None,
                'Description': None,
                'Projet': None,
                'Date de fin': None,
                'En attente de modération': False,
            },
            {
                'Titre': "test 2",
                'Durée': "21 minutes",
                'Heures, en décimal': 0.35,
                'Distance': 21,
                'Date': (date.today() - timedelta(1)).isoformat(),
                'Association': str(association),
                'Bénévole': str(user),
                'Catégorie': str(category),
                'Niveau': str(level),
                'Description': 'mise sous plis des convocations',
                'Projet': str(project),
                'Date de fin': (date.today() + timedelta(1)).isoformat(),
                'En attente de modération': True,
            },
        ]

    def test_post_create_update_delete(
        self, client, association, category, manager, user, foreign
    ):
        client.force_login(user)

        # 1. on fetch les benevalos de l'asso
        url = reverse('base:association:benevalo:index', args=[association.id])
        response = client.get(url)
        assert response.status_code == 200
        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])

        # 2. on créé un benevalo dans l'asso
        url = (
            bs(response.content, 'html.parser')
            .find(class_='table-buttons')
            .find('a')
            .attrs['href']
        )
        data = {
            'user': user.id,
            'title': 'benevalo de test',
            'duration_1': '42',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200
        assert count_text_in_content(response, "benevalo de test")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "« benevalo de test » a été ajouté" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "benevalo de test" in str(table_row[-1])

        # 3. on tente l'update
        url = (
            bs(response.content, 'html.parser')
            .find('tbody')
            .find_all('a')[-1]
            .attrs['href']
        )
        data = {
            'user': foreign.id,
            'title': 'benevalo de lutte',
            'duration_1': '42',
            'execution_date': date.today(),
            'association': association.id,
            'category': category.id,
        }
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        assert count_text_in_content(
            response, "Ce choix ne fait pas partie de ceux disponibles."
        )
        execution_date = (
            bs(response.content, 'html.parser')
            .find(id='id_execution_date')
            .attrs['value']
        )
        assert execution_date == date.today().strftime('%Y-%m-%d')

        # 4. Bien reçu, on retente en intégrant le foreign user
        association.engagement_set.create(
            user=foreign, role=association.role_set.get_benevole()
        )
        response = client.post(url, data, follow=True)

        assert not count_text_in_content(response, "benevalo de test")
        assert count_text_in_content(response, "benevalo de lutte")
        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été modifié" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "benevalo de lutte" in str(table_row[-1])

        # 5. on delete
        response = client.get(url, follow=True)
        url = (
            bs(response.content, 'html.parser')
            .find('a', string=RE_DELETE_BTN)
            .attrs['href']
        )
        data = {'confirm': True}
        response = client.post(url, data, follow=True)
        assert response.status_code == 200

        messages = [m.message for m in get_messages(response.wsgi_request)]
        assert len(messages) == 1
        assert "a été supprimé" in messages[0]

        table_row = (
            bs(response.content, 'html.parser').find('tbody').find_all('tr')
        )
        assert len(table_row) == 1
        assert "Aucune donnée disponible." in str(table_row[-1])


@pytest.mark.django_db
class TestEngagedBenevalo:
    @pytest.mark.parametrize(
        'use, expected',
        [(None, 404), ('user', 200), ('staff', 404), ('foreign', 404)],
    )
    def test_get_index(
        self,
        association,
        client,
        user,
        manager,
        superuser,
        foreign,
        use,
        expected,
    ):
        if use == 'staff':
            client.force_login(superuser)
        elif use == 'foreign':
            client.force_login(foreign)
        elif use:
            client.force_login(user)

        response = client.get(
            reverse(
                'base:association:engagement:benevalo',
                args=[association.id, association.get_engagement(user).id],
            )
        )
        assert response.status_code == expected


@pytest.mark.django_db
class TestStatsPage:
    def test_asso_count(self, client):
        user_email = "test@email.com"
        # On crée 5 assos
        objs = [Association(name="Asso {}".format(i)) for i in range(0, 5)]
        Association.objects.bulk_create(objs)
        # On crée le superuser pour accéder au contenu de la page
        User.objects.create_user(
            email=user_email,
            password="test",
            is_superuser=True,
        )
        response = client.post(
            reverse('accounts:login'),
            data={'username': user_email, 'password': 'test'},
            follow=True,
        )
        response = client.get(reverse('base:statistics'))
        # On vérifie que la page indique le bon chiffre
        assert "5 associations" in response.content.decode()

    def test_user_count(self, client):
        user_email = "test@email.com"
        # On crée 20 simpleusers
        objs = [
            User(
                email="user{}@test.com".format(i),
                password="test",
            )
            for i in range(0, 20)
        ]
        User.objects.bulk_create(objs)
        # On crée le superuser pour accéder au contenu de la page
        User.objects.create_user(
            email=user_email,
            password="test",
            is_superuser=True,
        )
        response = client.post(
            reverse('accounts:login'),
            data={'username': user_email, 'password': 'test'},
            follow=True,
        )
        response = client.get(reverse('base:statistics'))
        # On vérifie que la page indique le bon chiffre
        # on doit trouver 21 utilisateurs (20 users + le superuser)
        assert "21 utilisateurs" in response.content.decode()

    def test_benevalo_count(
        self, user, client, association, engagement, category
    ):
        # On crée 3 benevalo
        duration1 = timedelta(hours=5, minutes=30)
        duration2 = timedelta(hours=3, minutes=15)
        duration3 = timedelta(hours=5)
        user.benevalo_set.create(
            title="test 1",
            duration=duration1,
            distance=21,
            execution_date=date(2019, 12, 5),
            association=association,
            category=category,
        )
        user.benevalo_set.create(
            title="test 2",
            duration=duration2,
            distance=21,
            execution_date=date(2019, 12, 28),
            association=association,
            category=category,
        )
        user.benevalo_set.create(
            title="test 3",
            duration=duration3,
            distance=21,
            execution_date=date(2020, 9, 30),
            association=association,
            category=category,
        )
        user_email = "test@email.com"
        # On crée le superuser pour accéder au contenu de la page
        User.objects.create_user(
            email=user_email,
            password='test',
            is_superuser=True,
        )
        response = client.post(
            reverse('accounts:login'),
            data={'username': user_email, 'password': 'test'},
            follow=True,
        )
        response = client.get(reverse('base:statistics'))
        # On formatte les durées des benevalo
        formatted_duration_2019 = format_duration(duration1 + duration2)
        formatted_duration_2020 = format_duration(duration3)
        formatted_total_duration = format_duration(
            duration1 + duration2 + duration3
        )
        # On vérifie que la page indique les bons chiffres
        # nombre de benevalo
        # total
        assert "3 actes de bénévolat" in response.content.decode()
        # en 2019
        assert "2 actes de bénévolat" in response.content.decode()
        # en 2020
        assert "1 actes de bénévolat" in response.content.decode()
        # durée des benevalo
        # totale
        assert formatted_total_duration in response.content.decode()
        # en 2019
        assert formatted_duration_2019 in response.content.decode()
        # en 2020
        assert formatted_duration_2020 in response.content.decode()
        # distance de benevalo
        # totale
        assert "63km parcourus" in response.content.decode()
        # en 2019
        assert "42km parcourus" in response.content.decode()
        # en 2020
        assert "21km parcourus" in response.content.decode()

    def test_benevalo_categories_repartition(
        self, client, association, user, engagement, category
    ):
        # création du superuser
        user_email = "test@email.com"
        User.objects.create_user(
            email=user_email,
            password="test",
            is_superuser=True,
        )
        response = client.post(
            reverse('accounts:login'),
            data={'username': user_email, 'password': 'test'},
            follow=True,
        )
        # Le compte pour category est à 0, on vérifie donc qu'il
        # n'est pas affiché
        response = client.get(reverse('base:statistics'))
        assert ('InstanceCategoryTest') not in response.content.decode()
        # On crée 4 benevalo un par un
        # On teste à chaque fois l'actualisation des valeurs
        duration = timedelta(hours=5)
        user.benevalo_set.create(
            title="test 1",
            duration=duration,
            distance=21,
            execution_date=date(2019, 12, 5),
            association=association,
            category=category,
        )
        response = client.get(reverse('base:statistics'))
        asserts = [
            'InstanceCategoryTest-total-benevolats-1',
            'InstanceCategoryTest-2019-benevolats-1',
            'InstanceCategoryTest-2019-assos-1',
            'InstanceCategoryTest-2019-benevoles-1',
            'InstanceCategoryTest-2019-duree-5',
            'InstanceCategoryTest-2019-distance-21',
        ]
        for assertion in asserts:
            assert assertion in response.content.decode()
        user.benevalo_set.create(
            title="test 2",
            duration=duration,
            distance=21,
            execution_date=date(2019, 12, 28),
            association=association,
            category=category,
        )
        response = client.get(reverse('base:statistics'))
        asserts = [
            'InstanceCategoryTest-total-benevolats-2',
            'InstanceCategoryTest-total-distance-42',
            'InstanceCategoryTest-2019-benevolats-2',
            'InstanceCategoryTest-2019-assos-1',
            'InstanceCategoryTest-total-assos-1',
            'InstanceCategoryTest-2019-benevoles-1',
            'InstanceCategoryTest-2019-duree-10',
            'InstanceCategoryTest-2019-distance-42',
        ]
        for assertion in asserts:
            assert assertion in response.content.decode()
        user.benevalo_set.create(
            title="test 3",
            duration=duration,
            distance=21,
            execution_date=date(2020, 9, 30),
            association=association,
            category=category,
        )
        response = client.get(reverse('base:statistics'))
        asserts = [
            'InstanceCategoryTest-total-benevolats-3',
            'InstanceCategoryTest-total-distance-63',
            'InstanceCategoryTest-2019-benevolats-2',
            'InstanceCategoryTest-2020-benevolats-1',
            'InstanceCategoryTest-2020-duree-5',
            'InstanceCategoryTest-total-duree-15',
            'InstanceCategoryTest-2019-distance-42',
            'InstanceCategoryTest-2020-distance-21',
        ]
        for assertion in asserts:
            assert assertion in response.content.decode()
        # On crée cette fois un bénévalo d'une autre catégorie
        acat = Category.objects.filter(instance_category=12)
        user.benevalo_set.create(
            title="test 3",
            duration=duration,
            distance=21,
            execution_date=date(2020, 9, 30),
            association=association,
            category=acat[0],
        )
        response = client.get(reverse('base:statistics'))
        asserts = [
            'InstanceCategoryTest-total-benevolats-3',
            'InstanceCategoryTest-2019-benevolats-2',
            'InstanceCategoryTest-2020-benevolats-1',
            'Autres activités-total-benevolats-1',
            'Autres activités-2020-benevolats-1',
            'Autres activités-2020-benevoles-1',
            'Autres activités-2020-assos-1',
            'Autres activités-2020-duree-5',
            'Autres activités-2020-distance-21',
        ]
        for assertion in asserts:
            assert assertion in response.content.decode()

    def test_association_activity_fields_and_groups_repartition(
        self,
        client,
        activity_field,
    ):
        # création du superuser
        user_email = "test@email.com"
        User.objects.create_user(
            email=user_email,
            password="test",
            is_superuser=True,
        )
        response = client.post(
            reverse('accounts:login'),
            data={'username': user_email, 'password': 'test'},
            follow=True,
        )
        # On va chercher le compte adequat, il devrait être à 0
        # Et le domaine d'activité ne devrait pas apparaître sur la page
        response = client.get(reverse('base:statistics'))
        assert "data: [0, 0, 0, 0, 0, 0, 0, 0, 0]" in response.content.decode()
        assert 'ActivitéTest-0' not in response.content.decode()
        Association.objects.create(
            name="AssoTest",
            description="""test""",
            activity_field=activity_field,
        )
        Association.objects.create(
            name="AssoTest2",
            description="""test""",
            activity_field=activity_field,
        )
        afield = ActivityField.objects.filter(id=1)
        Association.objects.create(
            name="AssoTest3",
            description="""test""",
            activity_field=afield[0],
        )
        # Après la création des assos, les compteurs devraient être incrémentés
        response = client.get(reverse('base:statistics'))
        assert "data: [1, 0, 0, 0, 0, 0, 0, 0, 2]" in response.content.decode()
        assert 'ActivitéTest-2' in response.content.decode()
        assert (
            'Association culturelle ou artistique (musique, danse photographie,'
            ' théâtre, lecture, écriture, arts plastiques…) y compris'
            ' enseignement d’activités culturelles-1'
        ) in response.content.decode()

    def test_stats_page_redirects_not_logged_users(self, client):
        response = client.get(reverse('base:statistics'))
        assert response.status_code == 302
        assert response.url == "/account/login/?next=/statistics/"

    def test_stats_page_displays_403_to_non_superusers(self, client):
        user_email = "test@email.com"
        User.objects.create_user(email=user_email, password="test")
        response = client.post(
            reverse('accounts:login'),
            data={'username': user_email, 'password': 'test'},
            follow=True,
        )
        response = client.get(reverse('base:statistics'))
        assert response.status_code == 403

    def test_superusers_access_stats_page(self, client):
        user_email = "test@email.com"
        User.objects.create_user(
            email=user_email,
            password="test",
            is_superuser=True,
        )
        response = client.post(
            reverse('accounts:login'),
            data={'username': user_email, 'password': 'test'},
            follow=True,
        )
        response = client.get(reverse('base:statistics'))
        assert "Statistiques globales" in response.content.decode()
