from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import PermissionDenied
from django.views.generic import TemplateView

import pytest
from cruditor.mixins import CruditorMixin

from ..html import fa_icon
from ..views import CruditorPageMixin, PageMixin
from . import count_text_in_content


class Page(PageMixin, TemplateView):
    title = "Page title"
    template_name = 'base.html'


class CruditorView(CruditorMixin, TemplateView):
    def get_title(self):
        return "Title"


class CruditorPage(CruditorPageMixin, CruditorView):
    title = "Page title"
    template_name = 'base.html'


class CruditorWithAddPage(CruditorPage):
    create_url = '/create/'


# TESTS
# ------------------------------------------------------------------------------


class TestPageMixin:
    @pytest.fixture(autouse=True)
    def setup(self, rf):
        self.request = rf.get('/')
        self.request.user = AnonymousUser()
        self.request.session = {}
        self.view = Page()
        self.view.request = self.request

    def test_header_title(self):
        assert self.view.get_header_title() == "Page title"
        self.view.header_title = "Header title"
        assert self.view.get_header_title() == "Header title"
        self.view.header_title = None
        assert self.view.get_header_title() is None

    def test_title(self):
        assert self.view.get_title() == "Page title"
        with pytest.raises(ValueError):
            self.view.title = None
            self.view.get_title()

    @pytest.mark.django_db
    def test_page_context(self):
        context = self.view.get_page_context()
        assert context['title'] == "Page title"
        assert context['header_title'] == "Page title"
        assert not any(context['instancemessages'])
        assert not context['titlebuttons']

    @pytest.mark.django_db
    def test_response(self):
        response = self.view.dispatch(self.request)
        assert response.status_code == 200
        assert 'page' in response.context_data
        assert count_text_in_content(response, "Page title")


class TestCruditorPageMixin:
    @pytest.fixture(autouse=True)
    def setup(self, rf):
        self.request = rf.get('/')
        self.request.user = AnonymousUser()
        self.request.session = {}
        self.view = CruditorPage()
        self.view.request = self.request

    def test_not_logged_in(self):
        response = self.view.dispatch(self.request)
        assert response.status_code == 302

    def test_permission_denied(self):
        class AuthenticatedUser(AnonymousUser):
            is_active = True
            is_authenticated = True

        self.request.user = AuthenticatedUser()
        self.view.staff_required = True

        with pytest.raises(PermissionDenied):
            self.view.dispatch(self.request)

    def test_title(self):
        assert self.view.get_title() == "Page title"
        self.view.title = None
        assert self.view.get_title() == "Title"

    @pytest.mark.django_db
    def test_header_title(self):
        context = self.view.get_context_data()
        assert context['page']['header_title'] == "Page title"

        self.view.header_title = "Other title"
        context = self.view.get_context_data()
        assert context['page']['header_title'] == "Other title"

    @pytest.mark.django_db
    def test_page_context(self):
        context = self.view.get_context_data()
        assert 'page' in context
        assert 'header_title' in context['page']
        assert all(k in context['page'] for k in context['cruditor'].keys())

    def test_create_tablebutton(self):
        view = CruditorWithAddPage()
        view.request = self.request
        assert view.get_tablebuttons() == [
            {
                'url': '/create/',
                'label': fa_icon('plus-circle', "Ajouter"),
                'button_class': 'btn-secondary',
            }
        ]

        view.create_tablebutton_label = "Add"
        assert view.get_tablebuttons()[0]['label'] == fa_icon(
            'plus-circle', "Add"
        )

    def test_tablebuttons_empty(self):
        assert self.view.get_tablebuttons() == []

        class TestPage(CruditorWithAddPage):
            def add_create_tablebutton(self):
                return False

        assert TestPage().get_tablebuttons() == []
